%define anolis_release 4

%ifarch aarch64 x86_64
%bcond_with lua
%else
%bcond_without lua
%endif

# Swtich to build with tongsuo or openssl
%bcond_without tongsuo

%if %{with tongsuo}
%global __requires_exclude ^(libssl\.so.*|libcrypto\.so.*)$
%global __brp_check_rpaths %{nil}
%endif

%global  _hardened_build     1
%global  nginx_user          tengine

Name:              tengine
Version:           3.0.0
Release:           %{anolis_release}%{?dist}

Summary:           A distribution of Nginx with some advanced features
Group:             System Environment/Daemons
# BSD License (two clause)
# http://www.freebsd.org/copyright/freebsd-license.html
License:           BSD
URL:               http://tengine.taobao.org
Source0:           http://tengine.taobao.org/download/%{name}-%{version}.tar.gz
Patch0:            0000-Fix-build-with-external-shared-tongsuo.patch
Patch1:            0001-Fix-compilation-failure-of-grpc-module-when-add-ngx_.patch

Source10:          %{name}.service
Source11:          %{name}.logrotate
Source12:          nginx.conf

# Fixes for resolv.conf detection and specification
# https://github.com/alibaba/tengine/pull/1547
# This patch allows to unconditionally specify location of /etc/resolv.conf,
# thus avoiding the file check for environments that might not yet have /etc/resolv.conf created;
# as well as allowing Tengine to have its own app-specific resolv.conf location customized
#Patch0001:         0001-fixes-for-resolv_conf-detection-and-specification.patch

%if %{without tongsuo}
BuildRequires:     openssl-devel
Requires:          openssl
%else
BuildRequires:     tongsuo
Requires:          tongsuo = %{installed_version_of tongsuo}
%endif
BuildRequires:     pcre2-devel
BuildRequires:     zlib-devel
BuildRequires:     jemalloc-devel
%if %{with lua}
BuildRequires:     luajit-devel lua-devel
%endif
BuildRequires:     systemd-rpm-macros

%if %{with lua}
Requires:          luajit lua
%endif

Provides:          webserver

Requires(post):    systemd
Requires(preun):   systemd
Requires(postun):  systemd

%if %{with lua}
%define lua_inc_path $(ls /usr/include | grep luajit)
%endif

%description
Tengine is a web server originated by Taobao. It is based on the Nginx
HTTP server and has many advanced features.

%prep
%setup -q %{name}-%{version}
%if %{with tongsuo}
%patch -P0 -p1
%endif
%patch -P1 -p1
sed -i 's#nginx#tengine#g' contrib/vim/*/nginx.vim

%build

# nginx does not utilize a standard configure script.  It has its own
# and the standard configure options cause the nginx configure script
# to error out.  This is is also the reason for the DESTDIR environment
# variable.
export DESTDIR=%{buildroot}
./configure \
    --prefix=%{_datadir}/%{name} \
    --sbin-path=%{_sbindir}/%{name} \
    --modules-path=%{_libdir}/%{name}/modules \
    --conf-path=%{_sysconfdir}/%{name}/nginx.conf \
    --error-log-path=%{_localstatedir}/log/%{name}/error.log \
    --http-log-path=%{_localstatedir}/log/%{name}/access.log \
    --http-client-body-temp-path=%{_localstatedir}/lib/%{name}/tmp/client_body \
    --http-proxy-temp-path=%{_localstatedir}/lib/%{name}/tmp/proxy \
    --http-fastcgi-temp-path=%{_localstatedir}/lib/%{name}/tmp/fastcgi \
    --http-uwsgi-temp-path=%{_localstatedir}/lib/%{name}/tmp/uwsgi \
    --http-scgi-temp-path=%{_localstatedir}/lib/%{name}/tmp/scgi \
    --pid-path=/run/%{name}.pid \
    --lock-path=/run/lock/subsys/%{name} \
    --user=%{nginx_user} \
    --group=%{nginx_user} \
    --with-debug \
    --with-compat \
    --with-http_ssl_module \
    --with-openssl-async \
    --with-http_stub_status_module \
    --with-http_realip_module \
    --with-http_auth_request_module \
    --with-http_addition_module \
    --with-http_dav_module \
    --with-http_gzip_static_module \
    --with-http_sub_module \
    --with-http_v2_module \
    --with-stream \
    --with-stream_ssl_module \
    --with-stream_ssl_preread_module \
    --with-threads \
    --with-http_secure_link_module \
    --with-http_gunzip_module \
    --without-mail_pop3_module \
    --without-mail_smtp_module \
    --without-mail_imap_module \
    --without-http_uwsgi_module \
    --without-http_scgi_module \
    --with-http_random_index_module \
    --add-module=modules/ngx_backtrace_module \
    --add-module=modules/ngx_http_reqstat_module \
    --add-module=modules/ngx_http_upstream_dynamic_module \
    --add-module=modules/ngx_debug_pool \
    --add-module=modules/ngx_http_slice_module \
    --add-module=modules/ngx_http_upstream_dyups_module \
    --add-module=modules/ngx_debug_timer \
    --add-module=modules/ngx_http_sysguard_module \
    --add-module=modules/ngx_http_concat_module \
    --add-module=modules/ngx_http_upstream_session_sticky_module \
    --add-module=modules/ngx_http_footer_filter_module \
    --add-module=modules/ngx_http_trim_filter_module \
    --add-module=modules/ngx_http_user_agent_module \
%if %{with lua}
    --add-module=modules/ngx_http_lua_module \
%endif
    --add-module=modules/ngx_http_upstream_check_module \
    --add-module=modules/ngx_slab_stat \
    --add-module=modules/ngx_http_proxy_connect_module \
    --add-module=modules/ngx_http_upstream_consistent_hash_module \
    --with-cc-opt="-g -Og -fPIE -fstack-protector-strong \
-Wformat \
-Werror=format-security \
-Wno-deprecated-declarations \
-fno-strict-aliasing \
-D_FORTIFY_SOURCE=2 \
--param=ssp-buffer-size=4 \
-DTCP_FASTOPEN=23 \
-fPIC \
%ifarch x86_64
-m64 -mtune=native \
%endif
-Wno-cast-function-type" \
    --with-ld-opt="-fPIE -fPIC -pie -Wl,-z,relro -Wl,-z,now" \
%if %{with lua}
    --with-luajit-inc="%{_includedir}/%{lua_inc_path}" \
    --with-luajit-lib="%{_libdir}" \
%endif
    --with-openssl-opt="-fPIC" \
    --with-zlib-opt='-O3 -fPIC' \
    --with-pcre \
    --with-pcre-opt='-fPIC' \
    --with-pcre-jit \
    --with-jemalloc \
%if %{with tongsuo}
    --with-openssl=/opt/tongsuo \
    --add-module=modules/ngx_tongsuo_ntls \
    --with-openssl-opt="enable-ntls" \
%endif
    --with-stream_sni

%make_build


%install
make install DESTDIR=%{buildroot} INSTALLDIRS=vendor

find %{buildroot} -type f -name .packlist -exec rm -f '{}' \;
find %{buildroot} -type f -name perllocal.pod -exec rm -f '{}' \;
find %{buildroot} -type f -empty -exec rm -f '{}' \;
find %{buildroot} -type f -iname '*.so' -exec chmod 0755 '{}' \;

install -p -D -m 0644 %{SOURCE10} %{buildroot}%{_unitdir}/%{name}.service
install -p -D -m 0644 %{SOURCE11} %{buildroot}%{_sysconfdir}/logrotate.d/%{name}
install -p -d -m 0755 %{buildroot}%{_sysconfdir}/%{name}/conf.d
install -p -d -m 0755 %{buildroot}%{_sysconfdir}/%{name}/default.d
install -p -d -m 0700 %{buildroot}%{_localstatedir}/lib/%{name}
install -p -d -m 0700 %{buildroot}%{_localstatedir}/lib/%{name}/tmp
install -p -d -m 0700 %{buildroot}%{_localstatedir}/log/%{name}
install -p -d -m 0755 %{buildroot}%{_datadir}/%{name}/html
install -p -m 0644 %{SOURCE12} %{buildroot}%{_sysconfdir}/%{name}
install -p -D -m 0644 man/nginx.8 %{buildroot}%{_mandir}/man8/%{name}.8

for i in ftdetect indent syntax; do
    install -p -D -m644 contrib/vim/${i}/nginx.vim %{buildroot}%{_datadir}/vim/vimfiles/${i}/%{name}.vim
done

%pre
getent group %{nginx_user} > /dev/null || groupadd -r %{nginx_user}
getent passwd %{nginx_user} > /dev/null || \
    useradd -r -d %{_localstatedir}/lib/nginx -g %{nginx_user} \
    -s /sbin/nologin -c "Tengine web server" %{nginx_user}
exit 0

%post
%systemd_post %{name}.service

%preun
%systemd_preun %{name}.service

%postun
%systemd_postun %{name}.service

%files
%license LICENSE
%doc CHANGES CHANGES.te README.markdown AUTHORS.te THANKS.te
%{_datadir}/%{name}/html/*
%{_sbindir}/%{name}
%{_datadir}/vim/vimfiles/ftdetect/%{name}.vim
%{_datadir}/vim/vimfiles/syntax/%{name}.vim
%{_datadir}/vim/vimfiles/indent/%{name}.vim
%{_mandir}/man8/%{name}.8*
%{_unitdir}/%{name}.service
%config(noreplace) %{_sysconfdir}/%{name}/fastcgi.conf
%config(noreplace) %{_sysconfdir}/%{name}/fastcgi.conf.default
%config(noreplace) %{_sysconfdir}/%{name}/fastcgi_params
%config(noreplace) %{_sysconfdir}/%{name}/fastcgi_params.default
%config(noreplace) %{_sysconfdir}/%{name}/koi-utf
%config(noreplace) %{_sysconfdir}/%{name}/koi-win
%config(noreplace) %{_sysconfdir}/%{name}/mime.types
%config(noreplace) %{_sysconfdir}/%{name}/mime.types.default
%config(noreplace) %{_sysconfdir}/%{name}/nginx.conf
%config(noreplace) %{_sysconfdir}/%{name}/nginx.conf.default
%config(noreplace) %{_sysconfdir}/%{name}/scgi_params
%config(noreplace) %{_sysconfdir}/%{name}/scgi_params.default
%config(noreplace) %{_sysconfdir}/%{name}/uwsgi_params
%config(noreplace) %{_sysconfdir}/%{name}/uwsgi_params.default
%config(noreplace) %{_sysconfdir}/%{name}/win-utf
%config(noreplace) %{_sysconfdir}/logrotate.d/%{name}
%attr(700,%{nginx_user},%{nginx_user}) %dir %{_localstatedir}/lib/%{name}
%attr(700,%{nginx_user},%{nginx_user}) %dir %{_localstatedir}/lib/%{name}/tmp
%attr(700,%{nginx_user},%{nginx_user}) %dir %{_localstatedir}/log/%{name}

# nginx filesystem
%dir %{_datadir}/%{name}
%dir %{_datadir}/%{name}/html
%dir %{_sysconfdir}/%{name}
%dir %{_sysconfdir}/%{name}/conf.d
%dir %{_sysconfdir}/%{name}/default.d


%changelog
* Mon Aug 28 2023 Funda Wang <fundawang@yeah.net> - 3.0.0-4
- Build with shared external tongsuo

* Mon Aug 28 2023 happy_orange <songnannan@linux.alibaba.com> - 3.0.0-3
- update the tongsuo tar

* Sun Aug 27 2023 Funda Wang <fundawang@yeah.net> - 3.0.0-2
- Add switch on tongsuo or openssl
- Fix build with bundled openssl

* Sun Aug 27 2023 Chunmei Xu <xuchunmei@linux.alibaba.com> - 3.0.0-1
- update to 3.0.0 and build with tongsuo

* Sat Apr 08 2023 Chunmei Xu <xuchunmei@linux.alibaba.com> - 2.3.3-4
- init for Anolis OS 23

* Mon Nov 15 2021 Weitao Zhou <zhouwt@linux.alibaba.com> - 2.3.3-3
- Use host resolv.conf to fix missing resolv.conf error at service start

* Thu Nov 04 2021 Tao Xu <tao.xt@alibaba-inc.com> - 2.3.3-2
- Enable async SSL mode build for CPU acceleration
- Fixes for resolv.conf detection and specification

* Mon Jun 21 2021 Weitao Zhou <zhouwtlord@gmail.com> - 2.3.3-1
- Init version of 2.3.3
